@@ -1,29 +1,34 @@
 use crate::mint_2d;
 use approx::ulps_eq;
-#[cfg(feature = "impl-rayon")]
+#[cfg(feature = "rayon")]
 use rayon::prelude::*;
 use std::cmp::Ordering;
 use std::marker::PhantomData;
 
-pub struct ConvexHull<
+pub struct ConvexHull<T>
+where
     T: num_traits::Float
         + std::fmt::Debug
         + approx::AbsDiffEq
         + approx::UlpsEq
         + Sync
-        + approx::AbsDiffEq<Epsilon = T>,
-> {
+        + approx::AbsDiffEq<Epsilon = T>
+        + approx::UlpsEq<Epsilon = T>
+        + approx::RelativeEq<Epsilon = T>,
+{
     pd: PhantomData<T>,
 }
 
-impl<
-        T: num_traits::Float
-            + std::fmt::Debug
-            + approx::AbsDiffEq
-            + approx::UlpsEq
-            + Sync
-            + approx::AbsDiffEq<Epsilon = T>,
-    > ConvexHull<T>
+impl<T> ConvexHull<T>
+where
+    T: num_traits::Float
+        + std::fmt::Debug
+        + approx::AbsDiffEq
+        + approx::UlpsEq
+        + Sync
+        + approx::AbsDiffEq<Epsilon = T>
+        + approx::UlpsEq<Epsilon = T>
+        + approx::RelativeEq<Epsilon = T>,
 {
     /// finds the point with lowest x
     fn find_lowest_x(linestring: &[mint::Point2<T>]) -> (usize, mint::Point2<T>) {
@@ -243,7 +248,11 @@ impl<
     pub fn graham_scan<'a, I>(input: I) -> mint_2d::LineString2<T>
     where
         I: IntoIterator<Item = &'a mint::Point2<T>>,
-        T: 'a + Sync + approx::AbsDiffEq<Epsilon = T>,
+        T: 'a
+            + Sync
+            + approx::AbsDiffEq<Epsilon = T>
+            + approx::UlpsEq<Epsilon = T>
+            + approx::RelativeEq<Epsilon = T>,
     {
         let mut input_points = input.into_iter().copied().collect::<Vec<mint::Point2<T>>>();
 
@@ -306,6 +315,8 @@ impl<
     /// # use linestring::mint_2d;
     /// # use linestring::mint_2d::convex_hull;
     /// # use rand::{Rng, SeedableRng};
+    /// # use approx::AbsDiffEq;
+    /// # use approx::UlpsEq;
     ///
     /// let mut rng = rand_chacha::ChaCha8Rng::seed_from_u64(38);
     /// let mut points = Vec::<mint::Point2<f32>>::new();
@@ -326,11 +337,16 @@ impl<
     /// let b = mint_2d::LineString2::<f32>::default().with_points(points);
     /// let b = convex_hull::ConvexHull::graham_scan(b.points().iter());
     ///
-    /// assert!(convex_hull::ConvexHull::contains(&a, &b));
-    /// assert!(!convex_hull::ConvexHull::contains(&b, &a));
+    /// assert!(convex_hull::ConvexHull::contains(&a, &b, f32::default_epsilon(), f32::default_max_ulps()));
+    /// assert!(!convex_hull::ConvexHull::contains(&b, &a, f32::default_epsilon(), f32::default_max_ulps()));
     ///```
-    #[cfg(not(feature = "impl-rayon"))]
-    pub fn contains(a: &mint_2d::LineString2<T>, b: &mint_2d::LineString2<T>) -> bool {
+    #[cfg(not(feature = "rayon"))]
+    pub fn contains(
+        a: &mint_2d::LineString2<T>,
+        b: &mint_2d::LineString2<T>,
+        epsilon: T::Epsilon,
+        max_ulps: u32,
+    ) -> bool {
         if a.len() <= 1 {
             return false;
         }
@@ -344,7 +360,9 @@ impl<
             for l in a.as_lines().iter().skip(step1).step_by(4) {
                 for step2 in 0..4 {
                     for p in b.points.iter().skip(step2).step_by(4) {
-                        if !Self::is_point_left_allow_collinear(&l.start, &l.end, p) {
+                        if !Self::is_point_left_allow_collinear_ulps(
+                            &l.start, &l.end, p, epsilon, max_ulps,
+                        ) {
                             //println!("The point {:?} is not left of {:?}", p, l);
                             return false;
                         }
@@ -385,7 +403,7 @@ impl<
     /// assert!(convex_hull::ConvexHull::contains(&a, &b, f32::default_epsilon(), f32::default_max_ulps()));
     /// assert!(!convex_hull::ConvexHull::contains(&b, &a, f32::default_epsilon(), f32::default_max_ulps()));
     ///```
-    #[cfg(feature = "impl-rayon")]
+    #[cfg(feature = "rayon")]
     pub fn contains(
         a: &mint_2d::LineString2<T>,
         b: &mint_2d::LineString2<T>,
@@ -436,7 +454,7 @@ impl<
     /// assert!(!convex_hull::ConvexHull::contains_point_exclusive(&hull, &[10.0,9.99999].into()));
     ///
     ///```
-    #[cfg(feature = "impl-rayon")]
+    #[cfg(feature = "rayon")]
     pub fn contains_point_exclusive(a: &mint_2d::LineString2<T>, p: &mint::Point2<T>) -> bool {
         if a.len() <= 1 {
             return false;
@@ -471,7 +489,7 @@ impl<
     /// assert!(convex_hull::ConvexHull::contains_point_inclusive(&hull, &[9.999,5.0].into()));
     /// assert!(convex_hull::ConvexHull::contains_point_inclusive(&hull, &[10.0,5.0].into()));
     ///```
-    #[cfg(feature = "impl-rayon")]
+    #[cfg(feature = "rayon")]
     pub fn contains_point_inclusive(a: &mint_2d::LineString2<T>, p: &mint::Point2<T>) -> bool {
         if a.len() <= 1 {
             return false;
