@@ -1807,9 +1807,18 @@ where
 {
     let a_sub_b = sub(a, b);
     let a_sub_p = sub(a, p);
-    let a_sub_p_cross_a_sub_b = cross_z(&a_sub_p, &a_sub_b);
-    (a_sub_p_cross_a_sub_b * a_sub_p_cross_a_sub_b)
-        / (a_sub_b.x * a_sub_b.x + a_sub_b.y * a_sub_b.y)
+    let dot = (a_sub_b.x * a_sub_p.x + a_sub_b.y * a_sub_p.y)
+        / (a_sub_b.x * a_sub_b.x + a_sub_b.y * a_sub_b.y);
+    if dot < T::zero() {
+        a_sub_p.x * a_sub_p.x + a_sub_p.y * a_sub_p.y
+    } else if dot > T::one() {
+        let b_sub_p = sub(b, p);
+        b_sub_p.x * b_sub_p.x + b_sub_p.y * b_sub_p.y
+    } else {
+        let a_sub_p_cross_a_sub_b = cross_z(&a_sub_p, &a_sub_b);
+        (a_sub_p_cross_a_sub_b * a_sub_p_cross_a_sub_b)
+            / (a_sub_b.x * a_sub_b.x + a_sub_b.y * a_sub_b.y)
+    }
 }
 
 /// Same as distance_to_line_squared<T> but it can be called when a-b might be 0.
@@ -1832,11 +1841,7 @@ where
         // give the point-to-point answer if the segment is a point
         distance_to_point_squared(a, p)
     } else {
-        let a_sub_b = sub(a, b);
-        let a_sub_p = sub(a, p);
-        let a_sub_p_cross_a_sub_b = cross_z(&a_sub_p, &a_sub_b);
-        (a_sub_p_cross_a_sub_b * a_sub_p_cross_a_sub_b)
-            / (a_sub_b.x * a_sub_b.x + a_sub_b.y * a_sub_b.y)
+        distance_to_line_squared(a, b, p)
     }
 }
 
