@@ -45,7 +45,7 @@ licenses /why-not-lgpl.html>.
 
 use crate::{nalgebra_2d, LinestringError};
 use ahash::AHashSet;
-use approx::ulps_eq;
+use approx::{ulps_eq, AbsDiffEq, RelativeEq, UlpsEq};
 use core::fmt;
 use std::cmp;
 use std::collections::BTreeMap;
@@ -55,14 +55,24 @@ use std::marker::PhantomData;
 #[derive(Clone, Copy)]
 pub struct SiteEventKey<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     pub pos: nalgebra::Point2<T>,
 }
 
 impl<T> SiteEventKey<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     pub fn new(x: T, y: T) -> Self {
         Self {
@@ -73,7 +83,12 @@ where
 
 impl<T> std::fmt::Debug for SiteEventKey<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_tuple("")
@@ -85,7 +100,12 @@ where
 
 impl<T> PartialOrd for SiteEventKey<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     #[inline(always)]
     fn partial_cmp(&self, other: &Self) -> Option<cmp::Ordering> {
@@ -103,7 +123,12 @@ where
 
 impl<T> Ord for SiteEventKey<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     /// It should be impossible for a !is_finite() number to be added to SiteEventKey
     #[inline(always)]
@@ -114,7 +139,12 @@ where
 
 impl<T> PartialEq for SiteEventKey<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     #[inline(always)]
     fn eq(&self, other: &Self) -> bool {
@@ -122,7 +152,15 @@ where
     }
 }
 
-impl<T> Eq for SiteEventKey<T> where T: nalgebra::RealField + Sync {}
+impl<T> Eq for SiteEventKey<T> where
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast
+{
+}
 
 /// A container struct that keeps track of the lines around a pivot point.
 /// It only stores the lines with highest x value left of pivot point, and lines with lowest x
@@ -130,7 +168,12 @@ impl<T> Eq for SiteEventKey<T> where T: nalgebra::RealField + Sync {}
 /// leaning towards pivot point have priority.
 struct MinMax<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     best_left: Option<T>,
     slope: MinMaxSlope<T>,
@@ -139,7 +182,12 @@ where
 
 impl<T> MinMax<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     fn new() -> Self {
         Self {
@@ -218,7 +266,12 @@ where
 
 struct MinMaxSlope<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     best_left: Option<T>, // slope
     candidates_left: Vec<usize>,
@@ -229,7 +282,12 @@ where
 
 impl<T> MinMaxSlope<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     fn new() -> Self {
         Self {
@@ -349,7 +407,12 @@ where
 ///
 pub struct SiteEvent<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     drop: Option<Vec<usize>>,
     add: Option<Vec<usize>>,
@@ -360,7 +423,12 @@ where
 
 impl<T> SiteEvent<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     pub(crate) fn with_intersection(i: &[usize]) -> Self {
         Self {
@@ -401,7 +469,12 @@ fn sweepline_intersection<T>(
     other: &nalgebra_2d::Line2<T>,
 ) -> Option<(T, T)>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     // line equation: y=slope*x+d => d=y-slope*x => x = (y-d)/slope
     let y1 = other.start.y;
@@ -433,7 +506,12 @@ where
 /// to take() them and make the borrow-checker happy.
 pub struct IntersectionData<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     // sweep-line position
     sweepline_pos: nalgebra::Point2<T>,
@@ -461,7 +539,12 @@ where
 
 impl<T> Default for IntersectionData<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     fn default() -> Self {
         Self {
@@ -480,7 +563,12 @@ where
 
 impl<T> IntersectionData<T>
 where
-    T: nalgebra::RealField + Sync,
+    T: nalgebra::RealField
+        + Sync
+        + AbsDiffEq<Epsilon = T>
+        + UlpsEq<Epsilon = T>
+        + RelativeEq<Epsilon = T>
+        + num_traits::cast::NumCast,
 {
     pub fn get_sweepline_pos(&self) -> &nalgebra::Point2<T> {
         &self.sweepline_pos
@@ -753,7 +841,29 @@ where
         })?;
 
         loop {
-            if let Some((key, event)) = site_events.pop_first() {
+            if let Some((key, event)) = {
+                #[cfg(feature = "map_first_last")]
+                {
+                    site_events.pop_first()
+                }
+                #[cfg(not(feature = "map_first_last"))]
+                {
+                    // emulate pop_first
+                    if let Some((first_key, _)) = site_events.iter().next() {
+                        let first_key = first_key.clone();
+                        let v = site_events.remove(&first_key).ok_or_else(|| {
+                            LinestringError::InternalError(format!(
+                                "Found a key to pop but could not remove the value. {}:{}",
+                                file!(),
+                                line!()
+                            ))
+                        })?;
+                        Some((first_key, v))
+                    } else {
+                        None
+                    }
+                }
+            } {
                 self.handle_event(
                     &key,
                     &event,
@@ -1007,7 +1117,7 @@ where
                 #[cfg(feature = "console_trace")]
                 print!("testing intersection between {} and {}: ", left_i, right_i);
 
-                if let Some(intersection_p) = left_l.intersection_point(&right_l) {
+                if let Some(intersection_p) = left_l.intersection_point(right_l) {
                     let intersection_p = intersection_p.single();
                     if !intersection_p.x.is_finite() || !intersection_p.y.is_finite() {
                         return Err(LinestringError::InvalidData(format!(
